<?php

/**
 * * Copyright © Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Webkul\SalesforceConnector\Controller\Adminhtml\System\Config\Connection;

use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\App\PageCache\Version;
use Magento\Framework\App\Cache\TypeListInterface;
use Magento\Framework\App\Cache\Frontend\Pool;

class Disconnect extends \Webkul\SalesforceConnector\Controller\Adminhtml\System\Config\Connection
{
    /**
     * @var JsonFactory
     */
    protected $resultJsonFactory;
    protected $writerInterface;
    protected $_curl;
    protected $_helper;
    protected $cacheTypeList;
    protected $cacheFrontendPool;
    /**
     * @param \Magento\Backend\App\Action\Context $context
     * @param \Webkul\Eshopsync\Helper\Connection $connection
     * @param \Webkul\Eshopsync\Helper\Data $helper
     * @param \Magento\Config\Model\ResourceModel\Config $resourceConfig
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     * @param \Magento\Framework\HTTP\Client\Curl $curl
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        JsonFactory $resultJsonFactory,
        \Magento\Framework\App\Config\Storage\WriterInterface $writerInterface,
        \Magento\Framework\HTTP\Client\Curl $curl,
        TypeListInterface $cacheTypeList,
        Pool $cacheFrontendPool,
        \Webkul\SalesforceConnector\Helper\Connection $helper
    ) {
        parent::__construct($context);
        $this->writerInterface = $writerInterface;
        $this->resultJsonFactory = $resultJsonFactory;
        $this->_curl = $curl;
        $this->_helper = $helper;
        $this->cacheTypeList = $cacheTypeList;
        $this->cacheFrontendPool = $cacheFrontendPool;
    }

    public function execute()
    {
        $this->_helper->revokeConnection();
        $this->messageManager->addSuccess(__('Connection Deleted successfully'));
        $resultRedirect = $this->resultRedirectFactory->create();
        $resultRedirect->setPath('admin/system_config/edit/section/sfConnect/');
        
        $this->cacheTypeList->cleanType('config');
        foreach ($this->cacheFrontendPool as $cacheFrontend) {
            $cacheFrontend->getBackend()->clean();
        }
        return $resultRedirect;
    }
}
